import 'dart:io';
import 'dart:math';
import 'package:esp_bluetooth_provisioning/models/BleDevice.dart';
import 'package:esp_bluetooth_provisioning/models/enums.dart';
import 'package:esp_bluetooth_provisioning/esp_bluetooth_provisioning.dart';
import 'package:esp_bluetooth_provisioning_example/POPDialog.dart';
import 'package:esp_bluetooth_provisioning_example/wifi_dialog.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'dart:async';

import 'package:flutter/services.dart';
import 'package:flutter_spinkit/flutter_spinkit.dart';
import 'package:permission_handler/permission_handler.dart';

void main() {
  runApp(MaterialApp(home: MyApp()));
}

class MyApp extends StatefulWidget {
  @override
  _MyAppState createState() => _MyAppState();
}

class _MyAppState extends State<MyApp> {
  String _platformVersion = 'Unknown';
  ScaffoldFeatureController<SnackBar, SnackBarClosedReason> snackbar;
  PermissionStatus _locationPermissionStatus = PermissionStatus.unknown;
  var _scaffoldKey = GlobalKey<ScaffoldState>();
  BluetoothState state;
  bool isScanned = false;
  bool isLoading = false;
  bool securityCheck = false;
  List<BleDevices> devices;
  String loading;
  String pin;
  StreamSubscription controller;

  @override
  void initState() {
    super.initState();
    devices = [];
    loading = "";
    pin = "";
    initPlatformState();
  }

  Future<void> initPlatformState() async {
    String platformVersion;
    // Platform messages may fail, so we use a try/catch PlatformException.
    try {
      platformVersion = await EspBleProvisioning.platformVersion;
    } on PlatformException {
      platformVersion = 'Failed to get platform version.';
    }

    // If the widget was removed from the tree while the asynchronous platform
    // message was in flight, we want to discard the reply rather than calling
    // setState to update our non-existent appearance.
    if (!mounted) return;

    setState(() {
      _platformVersion = platformVersion;
      print("Platform version : $_platformVersion");
      EspBleProvisioning().requestPermission();
      startListening();
    });
  }

  //*******Listen for bluetooth state***********
  startListening() async {
    print("** Listening **");
    EspBleProvisioning().getBluetoothStateStream().listen((state) async {
      print("State *********** $state");
      if (this.mounted)
        setState(() {
          this.state = state;
        });
      if (state == BluetoothState.ON) {
        if (snackbar != null) {
          snackbar.close();
        }
        print("Bluetooth ON");
      } else if (state == BluetoothState.OFF) {
        showSnackbar("Please turn on bluetooth.");
      }
    });
  }

  //*******Scan for bluetooth device***********
  scanBleDevice() {
    setState(() {
      isLoading = true;
      loading = "Scanning bluetooth devices";
    });
    controller = EspBleProvisioning().scanBluetoothDevice().listen((value) {
      setState(() {
        if (value.length > 0) {
          print("************DEVICE SCAN RESULT : $value");
          isScanned = true;
          isLoading = false;
          devices.clear();
          for (int i = 0; i < value.length; i++) {
            if (!devices.contains(BleDevices(value[i].name, value[i].id)))
              devices.add(BleDevices(value[i].name, value[i].id));
          }
        } else {
          isScanned = false;
          isLoading = false;
          showSnackbar("No devices found!");
        }
      });
    });
  }

  //*******Connect to bluetooth device***********
  connectBleDevice(String pin, int index) async {
    setState(() {
      print("Connect to Device**************** ${devices[index].name}");
      loading = "Connecting to device";
      isLoading = true;
    });

    controller = EspBleProvisioning().connectToBluetoothDevice(devices[index].name, pin).listen((value) async {
      setState(() {
        if (value == "success") {
          print(value);
          isLoading = false;
          devices.clear();
          print("************security check completed");
          _showDialog(context);
        } else {
          isScanned = true;
          isLoading = false;
        }
      });
    });

    return true;
  }

  //*******Provision device***********
  provisionDevice(String ssid, String password) async {
    setState(() {
      isLoading = true;
      loading = "Provisioning device";
    });
    print("***********START WIFI PROVISIONING");
    controller = EspBleProvisioning().provisionDevice(ssid, password).listen((value) async {
      print("FrontEnd : $value");
      setState(() {
        isLoading = false;
        isScanned = false;
        devices.clear();
        print("Provision Status ----------> $value");
        messageDialog(value, context);
      });
    });
    return true;
  }

  void showSnackbar(message) {
    _scaffoldKey.currentState.showSnackBar(SnackBar(
      content: Text(message.toString()),
      action: SnackBarAction(
        label: 'Dismiss',
        onPressed: () {
          _scaffoldKey.currentState.removeCurrentSnackBar();
        },
      ),
    ));
  }

  void _showPOPDialog(BuildContext _context, int index) {
    showDialog(
        context: _context,
        builder: (BuildContext context) {
          return POPDialog(
            pin: "",
            onSubmit: (pin) async {
              print('pin = $pin');
              connectBleDevice(pin, index);
            },
          );
        });
  }

  void _showDialog(BuildContext _context) {
    showDialog(
        context: _context,
        builder: (BuildContext context) {
          return WifiDialog(
            wifiName: "",
            onSubmit: (ssid, password) async {
              print('ssid =$ssid, password = $password');
              Navigator.pop(context);
              provisionDevice(ssid, password);
            },
          );
        });
  }

  messageDialog(String message, BuildContext _context) {
    showDialog(
      barrierDismissible: true,
      context: _context,
      builder: (BuildContext context) {
        Future.delayed(Duration(seconds: 5), () {
          Navigator.of(context).pop(true);
        });
        return AlertDialog(
          shape: RoundedRectangleBorder(
            borderRadius: BorderRadius.circular(15.0),
          ),
          content: Stack(
            overflow: Overflow.visible,
            children: <Widget>[
              StatefulBuilder(
                builder: (BuildContext context, StateSetter alertState) {
                  return Container(
                      child: Column(
                    mainAxisAlignment: MainAxisAlignment.center,
                    crossAxisAlignment: CrossAxisAlignment.center,
                    mainAxisSize: MainAxisSize.min,
                    children: <Widget>[
                      Padding(
                        padding: EdgeInsets.only(left: 8.0),
                        child: Text(
                          message,
                          textAlign: TextAlign.center,
                          style: TextStyle(
                            fontFamily: "Montserrat",
                            fontSize: 16.0,
                            color: Colors.black,
                          ),
                        ),
                      ),
                      Padding(
                        padding: const EdgeInsets.only(top: 16),
                        child: Container(
                          decoration: BoxDecoration(
                            borderRadius: BorderRadius.circular(15),
                          ),
                          child: ElevatedButton(
                            onPressed: () {},
                            child: Text(
                              "Ok",
                              style: TextStyle(fontSize: 15, color: Colors.white),
                            ),
                          ),
                        ),
                      ),
                    ],
                  ));
                },
              ),
            ],
          ),
        );
      },
    );
  }

  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      home: Scaffold(
          key: _scaffoldKey,
          appBar: AppBar(
            title: const Text('Esp Ble Provisioning'),
          ),
          body: !isLoading
              ? isScanned
                  ? Container(
                      padding: EdgeInsets.all(16),
                      child: Column(
                        children: [
                          Text(
                            "Select a device for provisioning",
                            style: TextStyle(color: Colors.black, fontWeight: FontWeight.bold, fontSize: 18),
                          ),
                          Padding(
                            padding: const EdgeInsets.only(top: 16.0),
                            child: ListView.builder(
                                shrinkWrap: true,
                                itemCount: devices.length,
                                itemBuilder: (_context, index) {
                                  return GestureDetector(
                                      onTap: () {
                                        _showPOPDialog(context, index);
                                      },
                                      child: Container(
                                        width: double.infinity,
                                        child: Column(
                                          mainAxisAlignment: MainAxisAlignment.start,
                                          crossAxisAlignment: CrossAxisAlignment.start,
                                          children: [
                                            Padding(
                                              padding: const EdgeInsets.all(4.0),
                                              child: Text(
                                                devices[index].name,
                                                style: TextStyle(fontSize: 16, color: Colors.black),
                                              ),
                                            ),
                                            Padding(
                                              padding: const EdgeInsets.all(4.0),
                                              child: Text(
                                                devices[index].id,
                                                style: TextStyle(fontSize: 14, color: Colors.grey),
                                              ),
                                            ),
                                          ],
                                        ),
                                      ));
                                }),
                          ),
                        ],
                      ))
                  : Container(
                      height: double.infinity,
                      child: Center(
                        child: ElevatedButton(
                          onPressed: () => {
                            scanBleDevice()},
                          child: Text("Scan for bluetooth device"),
                        ),
                      ),
                    )
              : Align(
                  alignment: Alignment.center,
                  child: Container(
                    padding: EdgeInsets.only(top: 250),
                    color: Colors.white,
                    child: Center(
                      child: Column(
                        children: [
                          SpinKitRipple(color: Theme.of(context).textSelectionColor),
                          Padding(
                            padding: const EdgeInsets.all(8.0),
                            child: Text(
                              loading,
                              style: TextStyle(
                                fontSize: 14,
                              ),
                            ),
                          )
                        ],
                      ),
                    ),
                  ),
                )),
    );
  }
}

class BleDevices {
  String name;
  String id;

  BleDevices(this.name, this.id);

  @override
  String toString() {
    return '{ ${this.name}, ${this.id} }';
  }
}
